#ifndef DATAPORT_CPP
#define DATAPORT_CPP

// 'data_port' class. Provides convenient sequential read and write access to a data block.
// Optimized for 1, 2 and 4-byte requests and uploads. Use with care, since it has no
// safety features.
//
// Dmitry Yulmukhametov, Nov. 1999.

#include "basedef.h"
#include <mem.h>

class data_port {
	byte* data_buf;
	int pos;
	public:
		data_port(byte* data) : data_buf(data),pos(0) {}
		dword request(int sz); // get a small mem. block (1, 2 or 4 bytes).
		byte* request(byte* dest,int len); // get an arbitrary size mem.block;
		byte* request_addr(int len); // don't copy the block, just return it's address.
		void upload(dword src,int sz);
		void upload(byte* src,int len);
		void seek(int ofs);
};

dword data_port::request(int sz) {
	dword ret_value=0;
	switch (sz) {
		case 1: ret_value=(byte&)data_buf[pos]; break;
		case 2: ret_value=(word&)data_buf[pos]; break;
		case 4: ret_value=(dword&)data_buf[pos]; break;
	}
	pos+=sz;
	return ret_value;
}

byte* data_port::request(byte* dest,int len) {
	byte* ret_addr=(byte*)memcpy(dest,&data_buf[pos],len);
	pos+=len;
	return ret_addr;
}

byte* data_port::request_addr(int len) {
	byte* ret_addr=&data_buf[pos];
	pos+=len;
	return ret_addr;
}

void data_port::upload(dword src,int sz) {
	switch (sz) {
		case 1: (byte&)data_buf[pos]=src; break;
		case 2: (word&)data_buf[pos]=src; break;
		case 4: (dword&)data_buf[pos]=src; break;
	}
	pos+=sz;
}

void data_port::upload(byte* src,int len) {
	memcpy(&data_buf[pos],src,len);
	pos+=len;
}

void data_port::seek(int ofs) {
	pos+=ofs;
}

#endif