#ifndef INPUT_CPP
#define INPUT_CPP

#include "basedef.h"
#include "base.cpp"
#include <fstream.h>
#include <string.h>

word line_num=0;

class input : public ifstream {
	static int is_special(char c) {return (c==',' || c==':' || c==';' || c=='[' || c==']' || c=='"' || c=='$' || c==EOF);}
	static int is_stop(char c) {return is_special(c) || c==' ' || c=='\t' || c=='\n';}
	public:
		input(const char* filename) : ifstream(filename) {}
		int skip_until(char delim);
		void skip_ws(); // skips ' ','\t','\n'.
		int get_string(char* buf,int max,char delim,int accept_dbl_delim);
		int get_token(char* buf,int max); //  Returns zero on success
		int get_num(dword& buf,int base); // returns zero on success.
		int get_char(char); //returns zero on success.
		int get_dir(rec_t& buf);
		word get_line_num() {return line_num;}
};

int input::skip_until(char delim) {
	char c;
	while (1) {
		c=get();
		if (c==delim) break;
		if (c=='\n') line_num++;
		if (c==EOF) return 1;
	}
	return 0;
}

void input::skip_ws() {
	char c;
	while (1) {
		c=get();
		if (c==' ' || c=='\t') continue;
		if (c=='\n') {line_num++; continue;}
		if (c=='/') {ignore(0x7FFF,'\n'); line_num++; continue;}
		break;
	}
	putback(c);
}

int input::get_string(char* buf,int max,char delim,int accept_dbl_delim) {
	char c;
	int i=0;
	while (1) {
		c=get();
		if (c=='\n') line_num++;
		if (c==EOF || i==max) break;
		if (c==delim) {
			if (!accept_dbl_delim || peek()!=delim) break;
			get();
		}
		buf[i++]=c;
	}
	buf[i]='\0';
	return (c!=delim);
}

int input::get_token(char* buf,int max) {
	char c;
	int i=0;
	skip_ws();
	if (is_digit(peek(),10)) return 1;
	while (1) {
		c=get();
		if (c=='\n') line_num++;
		if (is_stop(c)) break;
		if (i==max) break;
		buf[i++]=c;
	}
	putback(c);
	if (i==0) return 1;
	buf[i]='\0';
	return 0;
}

int input::get_num(dword& buf,int base) {
	skip_ws();
	if (!is_digit(peek(),base)) return 1;
	buf=0;
	char c;
	while (1) {
		c=get();
		if (c=='\n') line_num++;
		if (is_stop(c)) break;
		if (!is_digit(c,base)) return 1;
		buf=buf*base+char_to_digit(c);
	}
	putback(c);
	return 0;
}

int input::get_char(char c) {
	skip_ws();
	return (get()!=c);
}

int input::get_dir(rec_t& buf) {
	char s[max_dir_len];
	if (skip_until('[')) return 1;
	get(s,max_dir_len,']');
	if (get()!=']') return 1;
	for (buf=0;buf<rec_t_cnt+1;buf++)
		if (!strcmp(s,directive[buf]))
			break;
	return (buf==rec_t_cnt+1);
}


#endif