#ifndef LVECTOR_CPP
#define LVECTOR_CPP

#include "basedef.h"
#include "error.h"
#include <string.h>
#include <stdlib.h>

// Unique Labels array designed for strictly sequential fill. Add NULLs to represent empty labels.
class Labels {
	char** lname; // arbitrary length array of strings.
	const int a_size; // number of elements in the array.
	int add_pos;
	public:
		Labels(int sz) : a_size(sz),lname(new char*[sz]),add_pos(0) {}
		void generate(char* base_str); // generate a_size different labels: "<base_str>number".
		int find(char* name,int limit); // find a label with matching name and value below limit. Returns limit, if label not found.
		err_c add_label(char* name); // Returns DUP_LABEL on error. DOES NOT do a strcpy().
		char* get_label(int k);
		int max_len();
		~Labels();
};

void Labels::generate(char* base_str) {
	for (add_pos=0;add_pos<a_size;add_pos++)
		if (base_str!=NULL) {
			lname[add_pos]=new char[max_lbl_len];
			strcpy(lname[add_pos],base_str);
			itoa(add_pos,lname[add_pos]+strlen(lname[add_pos]),10);
		}
		else
			lname[add_pos]=NULL;
}

int Labels::find(char* name,int limit) {
	int i;
	for (i=0;i<limit;i++)
		if (lname[i]!=NULL && !strcmp(name,lname[i]))
				break;
	return i;
}

err_c Labels::add_label(char* name) {
	if (find(name,add_pos)<add_pos) return DUP_LABEL;
	lname[add_pos++]=name;
	return NO_ERROR;
}

char* Labels::get_label(int k) {
	if (k>=0 && k<add_pos)
		return lname[k];
	return NULL;
}

Labels::~Labels() {
	for (int i=0;i<add_pos;i++)
		delete lname[i];
	delete lname;
}

int Labels::max_len() {
	int max=0;
	int buf;
	for (int i=0;i<a_size;i++)
		if ((buf= (lname[i]==NULL ? 0 : strlen(lname[i]))) > max)
			max=buf;
	return max;
}

#endif