#include <iostream.h>
#include <fstream.h>
#include <string.h>

#include "basedef.h"
#include "error.h"
#include "base.cpp"
#include "globdata.cpp"
#include "input.cpp"
#include "output.cpp"
#include "table.cpp"

enum Op_Mode {translating,extracting};
enum OS_Target {DOS,W95};

char* txt_fname; // the text I/O database file name.
char* exe_fname[2]={"orion2.exe","orion95.exe"};
char* str_fname="techname.lbx";
Op_Mode op_mode=translating; //default.
OS_Target target=W95; //default.

const char* ver_info="%version=0.21";
const char* ver_warning="/Don't edit the above line!";

err_c translate() {
	input txt(txt_fname);
	if (txt.bad()) return TXT_OPEN_FAILED;
	char* s=(char*)malloc(1000);
	txt.get(s,1000,'\n');
	if (strstr(s,ver_info)!=NULL)	version=v0_21; else version=v0_2;
	free(s);

	int fhand;
	if(fopen(exe_fname[target],fhand,0)) return BIN_OPEN_FAILED;
	for (int i=0;i<rec_t_cnt;i++) {
		safe_call(rdata[i]->load_bin(fhand,rec_ofs[i]));
	}
	fclose(fhand);

	for (i=0;i<rec_t_cnt;i++) {
		Table t(i);
		safe_call(t.load(txt));
	}
	txt.close();

	if(fopen(exe_fname[target],fhand,1)) return BIN_OPEN_FAILED;
	for (i=0;i<rec_t_cnt;i++) {
		safe_call(rdata[i]->store_bin(fhand,rec_ofs[i]));
	}
	fclose(fhand);

	long str_ofs=0x800;
	ofstream str(str_fname);
	if (str.bad()) return STR_OPEN_FAILED;
	long str_pre[]={0xFEAD0006,0,0x800};
	str.write((char*)&str_pre,12);
	for (i=12;i<0x800;i++)
		str.put(0);
	for (i=0;i<rec_t_cnt;i++) {
		str_ofs+=names[i]->store_str(str);
	}
	str.seekp(0x000C);
	str.write((char*)&str_ofs,4);
  str.flush();
	str.close();
	return NO_ERROR;
}

err_c extract() {
	int fhand;
	if(fopen(exe_fname[target],fhand,0)) return BIN_OPEN_FAILED;
	for (int i=0;i<rec_t_cnt;i++)
		safe_call(rdata[i]->load_bin(fhand,rec_ofs[i]));
	fclose(fhand);

	ifstream str(str_fname);
	if (str.bad()) return STR_OPEN_FAILED;
	str.seekg(0x800);
	for (i=0;i<rec_t_cnt;i++)
		safe_call(names[i]->load_str(str));
	str.close();

	for (i=0;i<rec_t_cnt;i++)
		labels[i]->generate(std_label[i]);

	output txt(txt_fname);
	txt.write(ver_info,strlen(ver_info));
	txt.put('\n');
	txt.write(ver_warning,strlen(ver_warning));
	for (i=0;i<rec_t_cnt;i++) {
		Table t(i);
		t.store(txt);
	}
	txt.flush();
	txt.close();
	return NO_ERROR;
}

// Error handler.
void error(err_c code) {
	const char* format_err_msg[]={"No error","Label or name expected","Wrong label","Colon expected","Quote expected",
																"Decimal number expected","Flags expected","Comma expected","Semicolon expected",
																"A label or a decimal number expected","Unknown label","Duplicate label"};
	const char* table_err_msg[]={"Directive expected","Wrong directive","[END] expected"};
	const char* out_of_mem="Out of memory";
	const char* load_fail[]={"Executable load failed","techname.lbx load failed"};
	const char* open_fail[]={"Could not open the file you specified","Could not open the executable","Could not open techname.lbx"};
	if (code==NO_ERROR)
		cout << "No error." << endl;
	else if (code<rec_err)
		cout << "Line " << line_num+1 << ": " << format_err_msg[code] << endl;
	else if (code<table_err)
		cout << "Line " << line_num+1 << ": " << table_err_msg[code-rec_err] << endl;
	else if (code<mem_err)
		cout << out_of_mem << endl;
		else if (code<load_err)
		cout << load_fail[code-mem_err] << endl;
	else
		cout << open_fail[code-load_err] << endl;
}

int init(int argc,char* argv[]) {
	const char* welcome="OCL 'MOO2' editor, v0.21 (c) Dmitry Yulmukhametov, Feb. 2000.\n";
	const char* call_format="Format: ocl_v021.exe [-e] [-d] <filename>\n";
	cout << welcome;
	if (argc<2 || argc>4) {
		cout << call_format;
		return 1;
	}
	txt_fname=argv[argc-1];
	for (int i=argc-2;i>0;i--) {
		if (!strcmp("-e",argv[i])) op_mode=extracting;
		else if (!strcmp("-d",argv[i])) target=DOS;
		else {
			cout << call_format;
			return 1;
		}
	}
	if (target==DOS) rec_ofs=(long*)rec_ofs_dos; else rec_ofs=(long*)rec_ofs_win;
	return 0;
}

int main(int argc,char* argv[]) {
	if (init(argc,argv)) return 1;
	init_glob();
	if (op_mode==translating) {
		cout << "Translation mode.\n";
		error(translate());
	}
	else {
		cout << "Extraction mode.\n";
		error(extract());
	}
	deinit_glob();
	return 0;
}